"""Module for defining motormongo datatype fields"""

from motormongo.fields.binary_field import BinaryField
from motormongo.fields.boolean_field import BooleanField
from motormongo.fields.datetime_field import DateTimeField
from motormongo.fields.embedded_document_field import EmbeddedDocumentField
from motormongo.fields.enum_field import EnumField
from motormongo.fields.field import Field
from motormongo.fields.float_field import FloatField
from motormongo.fields.geojson_field import GeoJSONField
from motormongo.fields.integer_field import IntegerField
from motormongo.fields.list_field import ListField
from motormongo.fields.reference_field import ReferenceField
from motormongo.fields.string_field import StringField
