from motormongo.abstracts.exceptions.document import (
    DocumentAggregationError,
    DocumentDeleteError,
    DocumentError,
    DocumentInsertError,
    DocumentNotFoundError,
    DocumentUpdateError,
    DocumentValidationError,
)
